/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ObservationSet.java,v 695470ee21f3 2010/06/07 22:11:15 fritz $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.observation;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.EventObject;
import org.openide.util.Lookup;
import it.tidalwave.util.As;
import it.tidalwave.util.Id;
import it.tidalwave.util.Initializer;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.bluebill.observation.Observation.Builder;

/***********************************************************************************************************************
 *
 * Represents a set of {@link Observation}.
 * The contained items need not to be related in any way.
 *
 * @author  Fabrizio Giudici
 * @version $Id: ObservationSet.java,v 695470ee21f3 2010/06/07 22:11:15 fritz $
 *
 **********************************************************************************************************************/
public interface ObservationSet extends Lookup.Provider, As
  {
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public class Event extends EventObject
      {
        @CheckForNull
        private final Observation observation;

        @CheckForNull
        private final Observation oldObservation;

        public Event (final @Nonnull ObservationSet observationSet,
                      final @CheckForNull Observation observation,
                      final @CheckForNull Observation oldObservation)
          {
            super(observationSet);
            this.observation = observation;
            this.oldObservation = oldObservation;
          }

        @CheckForNull
        public Observation getObservation()
          {
            return observation;
          }

        @CheckForNull
        public Observation getOldObservation()
          {
            return oldObservation;
          }
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public interface Listener
      {
        public void notifyObservationAdded (@Nonnull Event event);

        public void notifyObservationRemoved (@Nonnull Event event);

        public void notifyObservationChanged (@Nonnull Event event);

        public void notifyCleared (@Nonnull Event event);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public class ListenerSupport implements Listener
      {
        public void notifyObservationAdded (@Nonnull Event event)
          {
          }

        public void notifyObservationRemoved (@Nonnull Event event)
          {
          }

        public void notifyObservationChanged (@Nonnull Event event)
          {
          }

        public void notifyCleared (@Nonnull Event event)
          {
          }
      }

    /*******************************************************************************************************************
     *
     * Finds or create an object in the current {@code ObservationSet}.
     *
     * @param  clazz        the type of the object to find or create
     * @param  displayName  the display name of the object to find or create
     * @return              the object
     *
     ******************************************************************************************************************/
    @Nonnull
    public <T> T findOrCreate (@Nonnull Class<T> clazz,
                               @Nonnull String displayName);

    /*******************************************************************************************************************
     *
     * Finds or create an object in the current {@code ObservationSet}.
     *
     * @param  clazz        the type of the object to find or create
     * @param  displayName  the display name of the object to find or create
     * @param  initializer  an initializer to set the initial state of the
     *                      object in case it is created
     * @return              the object
     *
     ******************************************************************************************************************/
    @Nonnull
    public <T> T findOrCreate (@Nonnull Class<T> clazz,
                               @Nonnull String displayName,
                               @Nonnull Initializer<T> initializer);

    /*******************************************************************************************************************
     *
     * Finds an object given its id.
     *
     * @param  clazz              the type of the object to find
     * @param  id                 the id of the object to find
     * @throws NotFoundException  if the id doesn't match to any object
     * @return                    the object
     *
     ******************************************************************************************************************/
    @Nonnull
    public <T> T findById (@Nonnull Class<T> clazz, @Nonnull Id id)
      throws NotFoundException;

    /*******************************************************************************************************************
     *
     * Returns a {@link Finder} for performing a query of the given type.
     *
     * @param  clazz              the type of the objects to find
     * @return                    the {@code Finder}
     *
     ******************************************************************************************************************/
    @Nonnull
    public <T> Finder<T> find (@Nonnull Class<T> clazz);

    /*******************************************************************************************************************
     *
     * Returns a {@link Builder} for creating an {@link Observation}.
     *
     * @return  the {@code Builder}
     *
     ******************************************************************************************************************/
    @Nonnull
    public Observation.Builder createObservation();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void clear();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void addListener (@Nonnull Listener listener);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void removeListener (@Nonnull Listener listener);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    @Deprecated // TODO replace with findOrCreate()
    public Source createSource (@Nonnull String uri,
                                @Nonnull String displayName);
  }
